/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2020 IndiScale GmbH <info@indiscale.com>
 * Copyright (C) 2020 Timm Fitschen <t.fitschen@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */

'use strict';

var ext_applicable = function($, logger, is_in_view_port, load_config, getEntityPath, connection) {

  // TODO naming
  const previewReadyEvent = new Event("ext_bottom_line.preview.ready");

  const _css_class_preview_container = "caosdb-f-ext_bottom_line-container";
  const _css_class_preview_container_resolvable = "caosdb-f-ext_bottom_line-container-resolvable";
  const _css_class_preview_container_button = "caosdb-f-ext_bottom_line-container-button";

  var root_creator = async function (entity, creators) {
    for (let c of creators) {
        try {
            if (await c.is_applicable(entity)) {
                return c.create(entity);
            }
        } catch (err) {
            logger.error(err);
        }
    }
    return undefined;
  }

  var _set_content = function (container, content) {
    const _container = $(container);
    _container.empty();
    // TODO show/hide buttons for expanding the content
    var buttons = _container.siblings(`.${_css_class_preview_container_button}`);

    // toggle the buttons which expand the container
    if (content) {
        buttons.css({
            "visibility": "initial"
        });
        _container.append(content);
    } else {
        buttons.css({
            "visibility": "hidden"
        });
    }
  }

  var set_content = async function (container, content) {
    try {
      const wait = "Please wait...";
      _set_content(container, wait);
      const result = await content;
      _set_content(container, result);
      if (result) {
        // TODO event
        container.dispatchEvent(previewReadyEvent);
      }
    } catch (err) {
      logger.error(err);
      const err_msg = "An error occured while loading this content.";
      _set_content(container, err_msg);
    }
  }


  // TODO rename
  var root_handler = async function (entity, container, creators) {
    const _container = $(container);
    if (_container.hasClass(_css_class_preview_container_resolvable)) {
      _container.removeClass(_css_class_preview_container_resolvable);
      const content = root_creator(entity, creators);
      // TODO remove await?
      await set_content(container, content);
    }
  }

  var make_creators = function (creators) {
    const result = [];
    for (let c of creators) {
      result.push({
          id: c.id,
          is_applicable: typeof c.is_applicable === "function" ? c.is_applicable : eval(c.is_applicable),
          create: typeof c.create === "function" ? c.create : eval(c.create)
        });
    }
    return result;
  }


  var root_handler_trigger = function(root_handler) {
    var entities = $(".caosdb-entity-panel,.caosdb-entity-preview");
    for (let entity of entities) {

      // TODO viewport + 1000 px for earlier loading
      if (is_in_view_port(entity)) {
        root_handler(entity);
      }
    }
  }

  /**
   * Intialize the scroll watcher which listens on the scroll event of the
   * window and triggers the preview loading with a delay after the last
   * scroll event.
   *
   * @param {integer} delay - timeout in milliseconds after the last scroll
   *     event. After this timeout the trigger is called.
   * @param {function} trigger - the callback which is called.
   */
  var init_watcher = function(delay, trigger) {
    var scroll_timeout = undefined;
    $(window).scroll(() => {
        if (!scroll_timeout) {
            clearTimeout(scroll_timeout);
        }
        scroll_timeout = setTimeout(trigger, delay);
    });

    var preview_timeout = undefined;

    // init watcher on newly loaded entity previews.
    window.addEventListener(
      preview.previewReadyEvent.type,
      () => {
        if (!preview_timeout) {
          clearTimeout(scroll_timeout);
        }
        scroll_timeout = setTimeout(trigger, 100);
        return true;
      },
      true);

    // trigger for the first time
    trigger();
  };

  return {
    init_watcher: init_watcher,
    root_creator: root_creator,
    root_handler: root_handler,
    previewReadyEvent: previewReadyEvent,
    make_creators: make_creators,
    _css_class_preview_container: _css_class_preview_container,
    _css_class_preview_container_button: _css_class_preview_container_button,
    _css_class_preview_container_resolvable: _css_class_preview_container_resolvable,
  };

}($, log.getLogger("ext_applicable"), resolve_references.is_in_viewport_vertically, load_config, getEntityPath, connection);
