# How to add a module to CaosDB WebUI
The CaosDB WebUI is organized in modules which can easily be added and on a module basis enabled or disabled.

There are a few steps necessary to create a new module.

## Create the module file
Create a new file in `src/core/js` starting with `ext_`. E.g. `ext_flight_preview.js`. This file should define one function that wraps every thing and which is enabled at the bottom of the file:

```js
/*
 * ** header with license infoc
 * ...
 */

'use strict';

/**
 * description of the module ...
 *
 * @module ext_flight_preview
 * @version 0.1
 *
 * @requires somelibrary
 * (pass the dependencies as arguments)
 */
var ext_flight_preview = function (somelibrary) {

    var init = function (toolbox) {
        /* initialization of the module */
    }

    /**
     * doc string
     */
    var some_function = function (arg1, arg2) {
    }

    /* the main function must return the initialization of the module */
    return {
        init: init,
    };
//pass the dependencies as arguments here as well
}(somelibrary);

// this will be replaced by require.js in the future.
$(document).ready(function() {
    // use a variable starting with `BUILD_MODULE_` to enable your module
    // the build variable has to be enabled in the `build.properties.d/` directory.
    // Otherwise the module will not be activated.
    if ("${BUILD_MODULE_EXT_BOTTOM_LINE}" === "ENABLED") {
        caosdb_modules.register(ext_flight_preview);
    }
});
```
## Update xml
Add a section to `src/core/xsl/main.xsl` to include your new file.

```xsl
<xsl:element name="script">       
   <xsl:attribute name="src">         
     <xsl:value-of select="concat($basepath,'webinterface/${BUILD_NUMBER}/js/ext_sss_markdown.js')"/>             
   </xsl:attribute>     
</xsl:element>
```

## Add to index.html in test
If you have unittests (and you should), you need to add a line in :
`test/core/index.html`.

## Update the changelog

## Create a merge request